main.svelte-1oi270k{display:flex;flex-direction:row;text-align:left;padding-bottom:50px}.status.svelte-1oi270k{padding:15px}header.svelte-1oi270k{background:var(--mdc-theme-background);padding:1em;border-bottom:1px solid #e0e0e0}#side-menu.svelte-1oi270k{width:200px;border-right:1px solid #e0e0e0;height:calc(100vh - 74px)}#main.svelte-1oi270k{margin-left:20px;width:calc(100vw - 300px)}header.svelte-1oi270k{display:flex;flex-direction:row;justify-content:space-between;padding-right:50px}@media(min-width: 640px){main.svelte-1oi270k{max-width:none}}img.svelte-1oi270k{align-self:center}#bee-container.svelte-1wfszr0{width:50%;min-width:400px;padding:30px;height:75px}.table-container.svelte-1wfszr0{margin:0px auto;display:flex;flex-direction:column;width:fit-content}.results-header.svelte-1wfszr0{width:100%;display:flex;flex-direction:row;align-items:center;justify-content:space-between}.buttons-container.svelte-1wfszr0{display:flex;align-items:flex-end}#bee.svelte-1v5kw6w{width:50%;height:100px}.accordion-container.svelte-1s03elz{min-width:500px;width:100%}.tick.svelte-wr6453.svelte-wr6453{font-size:0.725em;font-weight:200}line.svelte-wr6453.svelte-wr6453,.tick.svelte-wr6453 line.svelte-wr6453{stroke:#aaa;stroke-dasharray:2}.tick.svelte-wr6453 text.svelte-wr6453{fill:#666}.tick.svelte-wr6453 .tick-mark.svelte-wr6453,.baseline.svelte-wr6453.svelte-wr6453{stroke-dasharray:0}.axis.snapTicks.svelte-wr6453 .tick:last-child text.svelte-wr6453{transform:translateX(3px)}.axis.snapTicks.svelte-wr6453 .tick.tick-0 text.svelte-wr6453{transform:translateX(-3px)}.tooltip.svelte-1d48k8j{position:absolute;width:150px;border:1px solid #ccc;font-size:13px;background:rgba(255, 255, 255, 0.85);transform:translate(-50%, -100%);padding:5px;z-index:15}.container.svelte-rj4j5n{display:flex;flex-direction:inline;flex-wrap:wrap;width:100%}.label.svelte-rj4j5n{font-size:9px;color:rgba(0, 0, 0, 0.5)}.output.svelte-rj4j5n{font-size:9px;color:rgba(255, 0, 0, 0.5)}.second_output.svelte-rj4j5n{font-size:9px;color:rgba(255, 0, 255, 0.5)}.box.svelte-rj4j5n{padding:10px}pre.langtag.svelte-1xjucv4{position:relative}pre.langtag.svelte-1xjucv4::after{content:attr(data-language);position:absolute;top:0;right:0;padding:1em;display:flex;align-items:center;justify-content:center;background:inherit;color:inherit;background:var(--hljs-background);color:var(--hljs-foreground);border-radius:var(--hljs-radius)}pre.langtag.svelte-1xjucv4{position:relative}pre.langtag.svelte-1xjucv4::after{content:attr(data-language);position:absolute;top:0;right:0;padding:1em;display:flex;align-items:center;justify-content:center;background:inherit;color:inherit;background:var(--hljs-background);color:var(--hljs-foreground);border-radius:var(--hljs-radius)}pre.langtag.svelte-1xjucv4{position:relative}pre.langtag.svelte-1xjucv4::after{content:attr(data-language);position:absolute;top:0;right:0;padding:1em;display:flex;align-items:center;justify-content:center;background:inherit;color:inherit;background:var(--hljs-background);color:var(--hljs-foreground);border-radius:var(--hljs-radius)}.layercake-container.svelte-vhzpsp,.layercake-container.svelte-vhzpsp *{box-sizing:border-box}.layercake-container.svelte-vhzpsp{width:100%;height:100%}div.svelte-1bu60uu,slot.svelte-1bu60uu{position:absolute;top:0;left:0}svg.svelte-6sm8ei{position:absolute;width:100%;height:100%;overflow:visible}svg.svelte-6sm8ei *{vector-effect:non-scaling-stroke}svg.svelte-u84d8d{position:absolute;top:0;left:0;overflow:visible}